"use strict";

this.author      = "cim"; 
this.copyright   = "2011-2015 cim."; 
this.licence = "CC-BY-SA 3.0";
this.name        = "Rescue Scenario 3"; 
this.description = "This script adds scenario 3 rescue mission."; 

//added long range map to the mission screens -spara-
//fixed description expand bug -spara-

this.scenarioID = "3";


this.startUp = function() {
// make sure Rescue Stations is loaded first
		if (worldScripts["Rescue Stations"].startUp) {
				worldScripts["Rescue Stations"].startUp();
		}
		worldScripts["Rescue Stations"].registerScenario(this.scenarioID);
}

this.missionAvailableCount = function() {
		if (Math.random() < 0.2) {
				return 1+this.missionAvailableCount();
		} else {
				return 1;
		}
}

this.missionAvailable = function() {

		if (Math.random() > 0.6) {
				return false; // usually available
		}
		if (missionVariables.rescuestation_reputation < 0) {
				return false; // need reasonable reputation
		}
		if (player.bounty > 50) {
				return false; // no fugitives
		}
/*		if (player.ship.equipmentStatus("EQ_FUEL_SCOOPS") != "EQUIPMENT_OK") {
				return false; // need a fuel scoop
		}
		if (player.ship.cargoSpaceCapacity < 1) {
				return false; // and somewhere to scoop to
		} */
		var nearby = this.targetSystems();
		if (nearby.length < 1) {
				return false; // no suitable nearby systems
		}
		return true;
}

this.targetSystems = function() {
		return SystemInfo.filteredSystems(this, function (other) {
				if (!((other.systemID !== system.info.systemID) && 
						(!other.sun_gone_nova) &&
						(system.info.distanceToSystem(other) > 7) &&
							(system.info.distanceToSystem(other) < 15))) {
						return false;
				} else {
						var route = system.info.routeToSystem(other);
						return (route && route.distance <= 25);
				}
		});
}

this.initMissionVariables = function() {
		missionVariables.rescuestation_startsystem = system.ID;
		missionVariables.rescuestation_startsystem_name = system.name;

		var nearby = this.targetSystems();
		var destination = nearby[Math.floor(nearby.length*Math.random())];

		missionVariables.rescuestation_destsystem = destination.systemID;
		missionVariables.rescuestation_destsystem_name = destination.name;

		var direction = worldScripts["Rescue Stations"].getDirection(system.info,destination);

		missionVariables.rescuestation_destsystem_desc = destination.name+" ("+expandDescription("[rescue_govname_"+destination.government+"]")+", "+Math.round(system.info.distanceToSystem(destination))+" LY "+direction+")";

		var timer = (2.3+Math.random())*(system.info.routeToSystem(destination).time);
		var deadline = clock.seconds+(timer*3600); // routeToSystem is in hours
// picked smallest number of jumps for timer, not quickest, to give a
// little flexibility

		missionVariables.rescuestation_deadline = deadline;
		missionVariables.rescuestation_deadline_text = clock.clockStringForTime(deadline);

		var reward = Math.floor(system.info.routeToSystem(destination).distance * 30);
		missionVariables.rescuestation_reward = reward;


		var danger1 = 0; 
		var danger2 = 0; // no ambush
		if (worldScripts["Rescue Stations"].scenariosWon("3") > 0) { // not the first time
				var ambush = (Math.floor(Math.random()*10))-destination.government;
				if (ambush > 0) { // almost always in Anarchy, down to 20% in Corporate
						danger1 = 2 + worldScripts["Rescue Stations"].dangerScale(512); 
						danger2 = 3 + worldScripts["Rescue Stations"].dangerScale(256); 
						if (destination.government <= 1) {
								danger2++;
								if (destination.government == 0) {
										danger1++;
								}
						}
				}
		}
		missionVariables.rescuestation_danger1 = danger1;
		missionVariables.rescuestation_danger2 = danger2;

}

this.rescue_shipDockedWithStation = function (rstation) {
		
		if (rstation.hasRole("rescue_station") && system.ID == missionVariables.rescuestation_startsystem && missionVariables.rescuestation_stage == 100) {
				if (clock.seconds < missionVariables.rescuestation_deadline) {
						missionVariables.rescuestation_stage = 1000;
				} else {
						missionVariables.rescuestation_stage = 1001;
				}
		}
}

this.rescue_missionScreenOpportunity = function() {
		if (worldScripts["Rescue Stations"].restoreDestination) {
			player.ship.targetSystem = worldScripts["Rescue Stations"].restoreDestination;
			delete worldScripts["Rescue Stations"].restoreDestination;
		}
		if (missionVariables.rescuestation_stage == 1) {
				mission.runScreen ({title: "Recover black box",
														messageKey: "rescue_scenario_3_mission_available",
														choicesKey: "rescue_scenario_nonurgent_mission_choices_map",
														model: "asteroid",
														background: "rrsfl_blank_bg.png"},
													 worldScripts["Rescue Scenario "+missionVariables.rescuestation_scenario].missionOfferDecision
													);
		} else if (missionVariables.rescuestation_stage == 1000) {
				var reward = missionVariables.rescuestation_reward;
				var debriefing = "rescue_scenario_3_mission_complete"
/*				if (missionVariables.rescuestation_danger1 > 0) {
						reward += 150;
						missionVariables.rescuestation_reward += 150;
						debriefing = "rescue_scenario_3_mission_complete_danger"
				} */
				player.credits += reward;

				mission.runScreen({title: "Black box recovered",
													 model: "rescue_blackbox",
													 messageKey: debriefing,
													 background: "rrsfl_blank_bg.png"});

				worldScripts["Rescue Stations"].missionSuccess();
		} else if (missionVariables.rescuestation_stage == 1001) {
				mission.runScreen({title: "Black box recovered",
													 messageKey: "rescue_scenario_3_mission_toolate",
													 background: "rrsfl_bg.png"});

				worldScripts["Rescue Stations"].failMission();
				
		}
}


this.rescue_shipWillLaunchFromStation = function (rstation) {
		if (missionVariables.rescuestation_stage == 2 && system.ID == missionVariables.rescuestation_destsystem && missionVariables.rescuestation_deadline > clock.seconds) {
				this.makeAsteroidField();
				player.consoleMessage("That black box must be somewhere around here.",10);
		}
}


this.rescue_populatingSystem = function() {
		if (missionVariables.rescuestation_stage == 2 && system.ID == missionVariables.rescuestation_destsystem && missionVariables.rescuestation_deadline > clock.seconds) {
				this.makeAsteroidField();
				player.consoleMessage("That black box must be somewhere around here.",10);
		} else if (missionVariables.rescuestation_stage == 2 && missionVariables.rescuestation_deadline <= clock.seconds) {
				player.consoleMessage("You have taken too long to find the black box.",10);			
				worldScripts["Rescue Stations"].failMission();
		}
}

/* Mission setup */

this.missionSuccess = function() {
		mission.unmarkSystem({system: missionVariables.rescuestation_startsystem, name: "RRS"});
		missionVariables.rescuestation_reputation += 10;
		if (missionVariables.rescuestation_danger1 > 0) {
				missionVariables.rescuestation_reputation += 3;
		}

		// nothing special		
}

this.failMission = function () {
		if (missionVariables.rescuestation_stage > 100) {
				mission.unmarkSystem({system: missionVariables.rescuestation_destsystem, name: "RRS"});
		} else {
				mission.unmarkSystem({system: missionVariables.rescuestation_startsystem, name: "RRS"});
		}
		missionVariables.rescuestation_reputation -= 5; 
		if (missionVariables.rescuestation_stage != 1001) {
				missionVariables.rescuestation_reputation -= 5; // bringing it back at all does reduce the penalty
		}
}

this.cleanupMission = function() {
		delete missionVariables.rescuestation_reward;
		delete missionVariables.rescuestation_deadline;
		delete missionVariables.rescuestation_deadline_text;
		delete missionVariables.rescuestation_startsystem;
		delete missionVariables.rescuestation_startsystem_name;
		delete missionVariables.rescuestation_destsystem;
		delete missionVariables.rescuestation_destsystem_name;
		delete missionVariables.rescuestation_danger1;
		delete missionVariables.rescuestation_danger2;
		player.ship.removeEquipment("EQ_RRS_CARGO_HOLDER1");
		mission.setInstructions(null);
}


this.missionOfferDecision = function (choice) {
		if (choice == "NO") {
				worldScripts["Rescue Stations"].refuseCurrentMission();
				worldScripts["Rescue Stations"].cleanupMission();
				missionVariables.rescuestation_stage = 0;
		} 
		
		else if (choice === "MAP") {
			worldScripts["Rescue Stations"].restoreDestination = player.ship.targetSystem;
			player.ship.targetSystem = missionVariables.rescuestation_destsystem;
				mission.runScreen ({title: "Recover black box",
														backgroundSpecial: "LONG_RANGE_CHART_QUICKEST"
														},
														worldScripts["Rescue Scenario "+missionVariables.rescuestation_scenario].rescue_missionScreenOpportunity
													);
		}
		
		else {
				missionVariables.rescuestation_stage = 2;
				mission.runScreen({title: "Mission accepted",
													 messageKey: "rescue_scenario_3_mission_accepted",
													 model: "rescue_blackbox",
													 background: "rrsfl_blank_bg.png"},
													function (choice) {
															mission.markSystem({system: missionVariables.rescuestation_destsystem, name: "RRS"});
															worldScripts["Rescue Stations"].acceptMission();
															worldScripts["Rescue Stations"].setMissionDetails("rescue_scenario_3_description");
													}
												 );
		}
}

/* Mission functions */

this.makeAsteroidField = function() {
		if (system.shipsWithPrimaryRole("rescue_blackbox").length > 0) {
				return; // already done
		}
		

		var fieldcentre = Vector3D(0,0,0.2+(Math.random()*0.3)).fromCoordinateSystem("wsu");
		fieldcentre.add(Vector3D.random(20E3));
		
		var asteroids = 150;
		var fieldradius = 20E3;
		for (var i=0;i<asteroids;i++) {
				var apos = fieldcentre.add(Vector3D.randomDirectionAndLength(fieldradius));
				system.addShips("asteroid",1,apos,1E3);
		}
		
		var bb = system.addShips("rescue_blackbox",1,fieldcentre,5E3);

		if (missionVariables.rescuestation_danger1 > 0) {
				system.addGroup("rrs_asteroid_ambusher",missionVariables.rescuestation_danger1+Math.floor(Math.random()*missionVariables.rescuestation_danger2),fieldcentre,15E3);
		} else {
				var bbpos = bb[0].position;
				// if there aren't pirates, then there might be a few cargo pods left
				if (Math.random() < 0.75) {
						system.addShips("alloy",1+Math.floor(Math.random()*3),bbpos,300);
				}
				if (Math.random() < 0.5) {
						system.addShips("barrel",1+Math.floor(Math.random()*2),bbpos,300);
				}
		}

}

this.collectBox = function() {
		missionVariables.rescuestation_stage = 100;
		mission.unmarkSystem({system: missionVariables.rescuestation_destsystem, name: "RRS"});
		mission.markSystem({system: missionVariables.rescuestation_startsystem, name: "RRS"});
		player.ship.awardEquipment("EQ_RRS_CARGO_HOLDER1");
		mission.setInstructions("Special Cargo: 1 TC Black box.", "Rescue Scenario 3");
}
