"use strict";

this.author      = "cim"; 
this.copyright   = "2011-2015 cim."; 
this.licence = "CC-BY-SA 3.0";
this.name        = "Rescue Scenario 6a"; 
this.description = "This script adds scenario 6 variant A rescue mission."; 

//added long range map to the mission screens -spara-
//made plague mine position fixed -spara-
//added background images to mission screens without model -spara-
//fixed description expand bug -spara-

this.scenarioID = "6a";

this.addedpirates = 0;
this.piratetimer = undefined;

this.startUp = function() {
// make sure Rescue Stations is loaded first
		if (worldScripts["Rescue Stations"].startUp) {
				worldScripts["Rescue Stations"].startUp();
		}
		worldScripts["Rescue Stations"].registerVariant("6",this.scenarioID);
}

this.targetSystems2 = function() {
		return SystemInfo.filteredSystems(this, function (other) {
				if (!((other.systemID !== system.info.systemID) && 
						(!other.sun_gone_nova) &&
						(other.description.indexOf("disease") != -1) &&
						(system.info.distanceToSystem(other) > 7) &&
							(system.info.distanceToSystem(other) < 30))) {
						return false;
				} else {
						var route = system.info.routeToSystem(other);
						return (route && route.distance <= 30);
				}
		});
}

this.targetSystems1 = function() {
		return SystemInfo.filteredSystems(this, function (other) {
				if (!((other.description.indexOf("disease") == -1) &&
							(!other.sun_gone_nova) &&
							(other.techlevel >= 11) &&
							(system.info.distanceToSystem(other) <= 15) && 
							(other.economy <= 1))) { // Rich/Avg industrials
								return false;
				} else {
						var route = system.info.routeToSystem(other);
						return (route && route.distance <= 15);
				}
		});
}


this.missionAvailable = function() {
		if (Math.random() > 0.3) {
				return false; // sometimes replace
		}
		if (missionVariables.rescuestation_reputation < 40) {
				return false; // not enough reputation
		}
		if (player.bounty > 50) {
				return false; // no fugitives
		}
		if (this.targetSystems1().length < 1) {
				return false; // no suitable systems
		}
		if (this.targetSystems2().length < 1) {
				return false; // no suitable systems
		}
		return true;
}

this.initMissionVariables = function() {
		missionVariables.rescuestation_system = system.ID;
		
		var nearby = this.targetSystems1();
		if (nearby.length == 0) {
				log(this.name,"No target systems found");
		}
		var destination1 = nearby[Math.floor(nearby.length*Math.random())];

		missionVariables.rescuestation_destsystem = destination1.systemID;
		missionVariables.rescuestation_destsystem_name = destination1.name;

		var direction = worldScripts["Rescue Stations"].getDirection(system.info,destination1);

		missionVariables.rescuestation_destsystem_desc = destination1.name+" ("+expandDescription("[rescue_govname_"+destination1.government+"]")+", "+Math.round(system.info.distanceToSystem(destination1))+" LY "+direction+")";

		var timer = (1.1+Math.random())*(system.info.routeToSystem(destination1).time);

		nearby = this.targetSystems2();
		var destination2 = nearby[Math.floor(nearby.length*Math.random())];
		missionVariables.rescuestation_destsystem2 = destination2.systemID;
		missionVariables.rescuestation_destsystem2_name = destination2.name;

		direction = worldScripts["Rescue Stations"].getDirection(system.info,destination2);

		missionVariables.rescuestation_destsystem2_desc = destination2.name+" ("+expandDescription("[rescue_govname_"+destination2.government+"]")+", "+Math.round(system.info.distanceToSystem(destination2))+" LY "+direction+")";

		timer += (1.1+Math.random())*(destination1.routeToSystem(destination2).time);
		var deadline = (3600*timer)+clock.seconds;

		missionVariables.rescuestation_deadline = deadline;
		missionVariables.rescuestation_deadline_text = clock.clockStringForTime(deadline);

		var reward = Math.floor(system.info.routeToSystem(destination2).distance * 100);
		missionVariables.rescuestation_reward = reward;

		missionVariables.rescuestation_danger = 2 + worldScripts["Rescue Stations"].dangerScale(192); 

		missionVariables.rescuestation_disease = expandDescription("[rescue_random_disease]");

}

this.rescue_shipDockedWithStation = function (rstation) {
		
		if (rstation.isMainStation && system.ID == missionVariables.rescuestation_destsystem && missionVariables.rescuestation_stage == 2) {
				if (clock.seconds < missionVariables.rescuestation_deadline) {
						missionVariables.rescuestation_stage = 90;
				} else {
						missionVariables.rescuestation_stage = 1002;
				}
		} else if (rstation.hasRole("rrs-plague-mine") && system.ID == missionVariables.rescuestation_destsystem2 && missionVariables.rescuestation_stage >= 100) {
				if (clock.seconds < missionVariables.rescuestation_deadline) {
						missionVariables.rescuestation_stage = 1000;
				} else {
						missionVariables.rescuestation_stage = 1001;
				}

		}

}

this.rescue_populatingSystem = function() {
		if (system.ID == missionVariables.rescuestation_destsystem2) {
			var i = 1;
			do {
				var mineloc = Vector3D(system.scrambledPseudoRandomNumber(i), system.scrambledPseudoRandomNumber(i + 1), system.scrambledPseudoRandomNumber(i + 2)).multiply(400E3);
				i++;
			} while (mineloc.magnitude() < 100E3);
				system.addShips("rrs-plague-mine",1,mineloc,1);
				system.addShips("asteroid", 30+Math.floor(Math.random()*30), mineloc, 40E3);
				system.addShips("miner",3,mineloc,20E3);
				var pirateCount = 1+Math.floor(Math.random()*missionVariables.rescuestation_danger);
				system.addGroup("pirate", pirateCount, [0,0,0], 15E3);

		}
}

this.rescue_shipExitedWitchspace = function() {
		if (this.piratetimer) {
				this.piratetimer.stop();
		}
		if (missionVariables.rescuestation_stage >= 100) {
				this.addedpirates = 0;
				this.additionalPirates();
		}
		if (system.ID == missionVariables.rescuestation_destsystem2) {
				if (!worldScripts["Rescue Ambience"].plagueActive()) {
						// there is an unseasonable plague here
						worldScripts["Rescue Ambience"].plagueSystem();
				}
		}
}

this.rescue_missionScreenOpportunity = function() {
		if (worldScripts["Rescue Stations"].restoreDestination) {
			player.ship.targetSystem = worldScripts["Rescue Stations"].restoreDestination;
			delete worldScripts["Rescue Stations"].restoreDestination;
		}
		if (missionVariables.rescuestation_stage == 1) {
				mission.runScreen ({title: "Medical supplies delivery",
														messageKey: "rescue_scenario_6a_mission_available",
														choicesKey: "rescue_scenario_nonurgent_mission_choices_map",
														model: "rescue_station",
														background: "rrsfl_blank_bg.png"},
													 worldScripts["Rescue Scenario "+missionVariables.rescuestation_scenario].missionOfferDecision
													);
		} else if (missionVariables.rescuestation_stage == 90) {
				mission.runScreen ({title: "Medical supplies delivery",
														messageKey: "rescue_scenario_6a_mission_progress",
														model: "1t-cargopod",
														background: "rrsfl_blank_bg.png"});
				missionVariables.rescuestation_stage = 100;
				mission.unmarkSystem({system: missionVariables.rescuestation_destsystem, name: "RRS"});
				mission.markSystem({system: missionVariables.rescuestation_destsystem2, name: "RRS"});

				player.ship.useSpecialCargo("RRS Medical Supplies");

		} else if (missionVariables.rescuestation_stage == 1000) {
				var reward = missionVariables.rescuestation_reward;
				player.credits += reward;

				mission.runScreen({title: "Delivery completed",
													 messageKey: "rescue_scenario_6a_mission_complete",
													 background: "rrsfl_bg.png"});

				worldScripts["Rescue Stations"].missionSuccess();
		} else if (missionVariables.rescuestation_stage == 1001) {
				var lateness = clock.seconds - missionVariables.rescuestation_deadline;
				if (lateness > 864000) {
						lateness = 864000; // more than 10 days late and it doesn't matter
				}
				missionVariables.rescuestation_reputation += 5; // partial mitigation
				missionVariables.rescuestation_reputation -= Math.floor(lateness/86400); // each extra day late makes it worse
				
				mission.runScreen({title: "Delivery completed",
													 messageKey: "rescue_scenario_6a_mission_toolate",
													 background: "rrsfl_bg.png"});

				worldScripts["Rescue Stations"].failMission();
		} else if (missionVariables.rescuestation_stage == 1002) {
				missionVariables.rescuestation_reputation -= 5; // serious error
				mission.runScreen({title: "We found someone else",
													 messageKey: "rescue_scenario_6a_mission_reallylate",
													 background: "rrsfl_bg.png"});
				worldScripts["Rescue Stations"].failMission();
		}
}



/* Mission setup */

this.missionSuccess = function() {
		mission.unmarkSystem({system: missionVariables.rescuestation_destsystem2, name: "RRS"});
		missionVariables.rescuestation_reputation += 15; 
		// nothing special		
		player.ship.removeAllCargo(); // remove the special cargo
}

this.failMission = function () {
		if (missionVariables.rescuestation_stage >= 100) {
				player.ship.removeAllCargo();  // remove the special cargo
		}
		mission.unmarkSystem({system: missionVariables.rescuestation_destsystem, name: "RRS"});
		mission.unmarkSystem({system: missionVariables.rescuestation_destsystem2, name: "RRS"});
		missionVariables.rescuestation_reputation -= 10; 

		var chance = 0.9;
		if (missionVariables.rescuestation_stage == 1001) {
				chance = 0.5; // better late than never?
		}

		worldScripts["Rescue News"].sendNews("rescue_scenario_6a_news",1,chance); // bad news
}

this.cleanupMission = function() {
		delete missionVariables.rescuestation_reward;
		delete missionVariables.rescuestation_system;
		delete missionVariables.rescuestation_danger;
		delete missionVariables.rescuestation_destsystem;
		delete missionVariables.rescuestation_destsystem_name;
		delete missionVariables.rescuestation_destsystem_desc;
		delete missionVariables.rescuestation_destsystem2;
		delete missionVariables.rescuestation_destsystem2_name;
		delete missionVariables.rescuestation_destsystem2_desc;
		delete missionVariables.rescuestation_deadline;
		delete missionVariables.rescuestation_deadline_text;
		if (this.piratetimer) {
				this.piratetimer.stop();
		}
}


this.missionOfferDecision = function (choice) {
		if (choice == "NO") {
				worldScripts["Rescue Stations"].refuseCurrentMission();
				worldScripts["Rescue Stations"].cleanupMission();
				missionVariables.rescuestation_stage = 0;
		}
		else if (choice === "MAP" || choice === "SWAP") {
			var options = {
				"1_SWAP" : "Swap destination",
				"2_RETURN" : "Return"
			};
			if (choice === "MAP") {
				worldScripts["Rescue Stations"].restoreDestination = player.ship.targetSystem;
				player.ship.targetSystem = missionVariables.rescuestation_destsystem;
			}
			else {
				if (player.ship.targetSystem === missionVariables.rescuestation_destsystem)
					player.ship.targetSystem = missionVariables.rescuestation_destsystem2;
				else
					player.ship.targetSystem = missionVariables.rescuestation_destsystem;
			}
			mission.runScreen ({
					title: "Medical supplies delivery",
					backgroundSpecial: "LONG_RANGE_CHART_QUICKEST",
					choices: options
				},
				function (choice) {
					if (choice === "1_SWAP") {
						worldScripts["Rescue Scenario "+missionVariables.rescuestation_scenario].missionOfferDecision("SWAP");
						return;
					}
					else {
						worldScripts["Rescue Scenario "+missionVariables.rescuestation_scenario].rescue_missionScreenOpportunity();
						return;
					}
				}
			);
		}
		else {
			missionVariables.rescuestation_stage = 2;
			mission.runScreen({
					title: "Transferring package",
					messageKey: "rescue_scenario_6a_mission_accepted",
					model: "rescue_blackbox"
				},
				function (choice) {
						worldScripts["Rescue Stations"].acceptMission();
						mission.markSystem({system: missionVariables.rescuestation_destsystem, name: "RRS"});
						worldScripts["Rescue Stations"].setMissionDetails("rescue_scenario_6a_description");
				}
			);
		}
}

		
/* Mission functions */



this.additionalPirates = function() {
		if (!player.ship.docked) {
				this.addedpirates++;
				var pirateCount = 1+Math.floor(Math.random()*missionVariables.rescuestation_danger);
				var addplace = player.ship.position.add(player.ship.vectorForward.multiply(50E3));
				system.addGroup("pirate", pirateCount, addplace, 10E3);
		}
		
		if (this.addedpirates <= missionVariables.rescuestation_danger) {
				if (system.ID == missionVariables.rescuestation_destsystem2) {
						var delay = 10*(1+system.info.government)+Math.floor(Math.random()*20);
				} else {
						var delay = (20*(4+system.info.government))+Math.floor(Math.random()*20);
				}
				if (this.piratetimer) {
						this.piratetimer.stop();
				}
				this.piratetimer = new Timer(this,this.additionalPirates,delay);
		}

}

